const path = require('path');
const Electron = require('electron');
const WindowManager = require('./windowManager');
const MainMenu = require('./mainMenu');
const spawn = require('child_process').spawn;
const config = require('./config');
const http = require('http');

/**
 * Application entry point.
 */
class App {
	/**
     * Initialize instance.
     */
	constructor() {
		// Compile switch
		global.DEBUG = true;
		if (DEBUG) {
			console.log('Initialize Application');
		}

		process.env.IS_DESKTOP = true;

		this._server = null;

		this.serverReady = false;

		/**
         * IPC module for main process.
         * @type {ipcMain}
         */
		this._ipc = require('electron').ipcMain;

		/**
         * The shell module provides functions related to desktop integration.
         * @type {shell}
         */
		this._shell = require('electron').shell;

		/**
         * Manage the window.
         * @type {WindowManager}
         */
		this._windowManager = new WindowManager(this);

	}

	/**
     * Get the IPC module.
     * @return {ipcMain} IPC module.
     */
	get ipc() {
		return this._ipc;
	}

	/**
     * Get the shell module.
     * @return {shell} IPC module.
     */
	get shell() {
		return this._shell;
	}

	/**
	* Get the window manager.
	*
	* @return {WindowManager} Instance of the window manager.
	*/
	get windowManager() {
		return this._windowManager;
	}

	/**
     * Occurs when a application launched.
     */
	onReady() {
		// after server start load config file
		config.loadUserConfig();
		// Check server version
		http.get(`http://localhost:${config.getUserConfig().port}/api/environment/info`, proxyRes => {
			proxyRes.setEncoding('utf8');
			let response = '';
			proxyRes
				.on('data', data => {
					response += data;
				})
				.on('end', () => {
					let data = JSON.parse(response).data;
					if ( data && data.version !== process.env.APP_VER ) {
						const errorDescription = 'Your Graphileon server version is not correct. Please try to close Graphileon Application and restart or reinstall';
						const options = {
					    type: 'warning',
					    buttons: [], // Empty Array shows just OK button which has index 0
					    defaultId: 2,
					    title: 'Graphileon',
					    message: 'Loading Graphileon App',
					    detail: errorDescription
						};

				  	Electron.dialog.showMessageBoxSync(options);
						console.log('The server version is not the same as the expected version');
					}
				})
		})
		.on('error', (err) => {
			// on connection error log the error to the console.
			console.log('Server Checking Error: ', err);
	    });
		// crreate window to load application
		const menu = Electron.Menu.buildFromTemplate(MainMenu.menu(this));
		Electron.Menu.setApplicationMenu(menu);
		this._windowManager.createNewWindow();
	}

	/**
	* Occurs when a window all closed.
	*/
	onWindowAllClosed() {
		if (DEBUG) {
			console.log('Quit');
		}
		Electron.app.quit();
	}

	/**
	 * remove user token when quit
	 */
	removeToken() {
		this._windowManager.removeToken();
	}

	/**
	 * in enterprise to stop local Graphileon Server
	 */
	stopServer() {
		if (this.serverReady) {
			this._server.kill();
		}
		this.serverReady = false;
	}

	/**
	 * in enterprise to start local Graphileon Server
	 */
	startServer() {
		this._server = spawn(config.serverPath, {
			cwd: path.resolve(__dirname).replace('app.asar', 'app.asar.unpacked')
		});

		this._server.stderr.on('data', (data) => {
		  	console.error(`Server Error: ${data}`);
		});

		this._server.stdout.on('data', (data) => {
			console.log(`Server Log: ${data}`);
			if ( data.toString().trim() === 'Server Started' ) {
				this.serverReady = true;
				this.onReady();
			}
		});

		this._server.on('close', (code) => {
			console.log(`child process exited with code ${code}`);
		});
	}


	licenseCheck() {

		http.get(config.getUserConfig().url.trim('/') + '/api/license/check?info=1', proxyRes => {
			proxyRes.setEncoding('utf8');
			let response = '';
			proxyRes
				.on('data', data => {
				   response += data;
				})
				.on('end', () => {
					this._windowManager.showAlert(JSON.parse(response).data.message, {headerText: 'License check', width: '540px', modalSize: 'modal-md'});
				});
		});
	}
}

const app = new App();

Electron.app.on('ready', () => {
	config.checkUserConfig();
	app.startServer();

	if (DEBUG) {
		console.log('Application is ready');
	}
});

Electron.app.on('quit', () => {
	app.removeToken();
	app.stopServer();
	if (DEBUG) {
		console.log('Application is quit');
	}
});

Electron.app.on('window-all-closed', () => {
	app.removeToken();
	if (DEBUG) {
		console.log('All of the window was closed.');
	}
	app.onWindowAllClosed();
});