const Electron = require('electron');
const config = require('./config');

const DOC_URL = 'http://docs.graphileon.com';
const NOTES_URL = 'http://docs.graphileon.com/interactor/Release_notes.html';

/**
 * Main menu.
 */
class MainMenu {
	/**
     * Create menu.
     *
     * @param {App} context Application instance.
     *
     * @return {Object[]} Menu.
     */
	static menu(context) {
		const templates = [
			MainMenu._menuEdit(context),
			MainMenu._menuView(),
			MainMenu._menuWindow(context),
			MainMenu._menuHelp(context)
		];

		if (DEBUG) {
			templates.push(MainMenu._menuDeveloper(context));
		}
		templates.unshift(MainMenu._menuApp(context));

		return templates;
	}

	/**
     * Create a menu of Application ( OS X only ).
     *
     * @return {Object} Menu data.
     */
	static _menuApp(context) {
		let template = {
			label: config.APP_NAME,
			submenu: [{
				label: 'About ' + config.APP_NAME,
				click: () => {
					context.windowManager.createAboutWindow();
				}
			},
			{
				type: 'separator'
			}]
		};
		if (process.platform === 'darwin') {
			/*{
				label: 'Services',
				role: 'services',
				submenu: []
			},
			{
				type: 'separator'
			},*/
			template.submenu.push({
				label: 'Hide ' + config.APP_NAME,
				accelerator: 'Command+H',
				role: 'hide'
			});
			template.submenu.push({
				label: 'Hide Others',
				accelerator: 'Command+Alt+H',
				role: 'hideothers'
			});
			template.submenu.push({
				label: 'Show All',
				role: 'unhide'
			});
			template.submenu.push({type: 'separator'});
		}
		template.submenu.push({
			label: 'License Check',
			click: () => {
				context.licenseCheck();
			}
		});
		template.submenu.push({
			label: 'Quit ' + config.APP_NAME,
			accelerator: (() => {
				return (process.platform === 'darwin' ? 'Command+Q' : 'Alt+F4');
			})(),
			click: () => { Electron.app.quit(); }
		});
		return template;
	}

	/**
     * Create a menu of View.
     *
     * @return {Object} Menu data.
     */
	static _menuView() {
		const templates = {
			label: 'View',
			submenu: [
				{ role: 'resetzoom' },
				{ role: 'zoomin' },
				{ role: 'zoomout' },
				{ type: 'separator' },
				{
					label: 'Toggle Full Screen',
					accelerator: (() => {
						return (process.platform === 'darwin' ? 'Ctrl+Command+F' : 'F11');
					})(),
					click: (item, focusedWindow) => {
						if (focusedWindow) {
							focusedWindow.setFullScreen(!(focusedWindow.isFullScreen()));
						}
					}
				}
			]
		};

		return templates;
	}

	/**
     * Create a menu of Edit.
     *
     * @return {Object} Menu data.
     */
	static _menuEdit(context) {
		const templates = {
			label: 'Edit',
			submenu: [
				{role: 'copy'},
				{role: 'cut'},
				{role: 'paste'},
				{role: 'delete'},
				{ role: 'selectall' },
				{ type: 'separator' }
			]
		};
		return templates;
	}

	/**
     * Create a menu of Window.
     *
     * @return {Object} Menu data.
     */
	static _menuWindow(context) {
		const templates = {
			label: 'Window',
			role: 'window',
			submenu: [{
				label: 'Reload',
				accelerator: 'CmdOrCtrl+R',
				click: (item, focusedWindow) => {
					if (focusedWindow) {
						focusedWindow.reload();
					}
				}
			},
			{
				label: 'New Window',
				accelerator: 'CommandOrControl+n',
				click: () => { context.windowManager.createNewWindow(); }
			},
			{type: 'separator'},
			{
				label: 'Minimize',
				accelerator: 'CmdOrCtrl+M',
				role: 'minimize'
			},
			{
				label: 'Close',
				accelerator: 'CmdOrCtrl+W',
				role: 'close'
			}
			]
		};

		if (process.platform === 'darwin') {
			templates.submenu.push({
				type: 'separator'
			});

			templates.submenu.push({
				label: 'Bring All to Front',
				role: 'front'
			});
		}

		return templates;
	}

	/**
     * Create a menu of Help.
     *
     * @return {Object} Menu data.
     */
	static _menuHelp() {
		return {
			label: 'Help',
			role: 'help',
			submenu: [
				{
					label: 'Documentation',
					click: () => {
						Electron.shell.openExternal(DOC_URL);
					}
				},
				{
					label: 'Release notes',
					click: () => {
						Electron.shell.openExternal(NOTES_URL);
					}
				}
			]
		};
	}

	/**
     * Create a menu of Help.
     *
     * @return {Object} Menu data.
     */
	static _menuDeveloper(context) {
		const templates = {
			label: 'Developer',
			role: 'developer',
			submenu: []
		};
		templates.submenu.push({
			label: 'Toggle Developer Tools',
			accelerator: (() => {
				return (process.platform === 'darwin' ? 'Alt+Command+I' : 'Ctrl+Shift+I');
			})(),
			click: (item, focusedWindow) => {
				if (focusedWindow) {
					focusedWindow.toggleDevTools();
				}
			}
		});

		return templates;
	}
}


module.exports = MainMenu;