const Electron = require('electron');
const path = require('path');
let config = require('./config');

/**
 * Manage the window.
 */
class WindowManager {
	/**
     * Initialize instance.
     *
     * @param {App} context Application context.
     */
	constructor(context) {
		/**
         * Application context.
         * @type {App}
         */
		this._context = context;

		/**
         * Collection of a managed window.
         * @type {Map.<String, BrowserWindow>}
         */
		this._windows = new Map();
	}

	/**
     * Reload the focused window, For debug.
     */
	reload() {
		const w = Electron.BrowserWindow.getFocusedWindow();
		if (w) {
			w.reload();
		}
	}

	/**
     * Switch the display of the developer tools window at focused window, For debug.
     */
	toggleDevTools() {
		const w = Electron.BrowserWindow.getFocusedWindow();
		if (w) {
			w.toggleDevTools();
		}
	}

	/**
     * Create a new window.
     *
     * @return {BrowserWindow} Created window.
     */
	createNewWindow() {
		let reloadCount = 5;

		const iconPath = path.join(__dirname, '/icons/icon.png'),
			w = new Electron.BrowserWindow({

				width: 1280,
				height: 870,
				minWidth: 800,
				minHeight: 600,
				resizable: true,
				show: false,
				icon: iconPath,
				backgroundColor: '#CCC',
				paintWhenInitiallyHidden: false,
				webPreferences: {
					zoomFactor: 0.75,
					nodeIntegration: true
				}
			});

		const id = w.id;

		w.on('closed', () => {
			if (DEBUG) {
				console.log('Window was closed, id = ' + id);
			}

			// Unregister
			this._windows.delete(id);
			if (this._windows.length === 0) {
				this.removeToken();
			}
		});

		const handleRedirectNewWindow = (e, url, frameName, disposition, options, additionalFeatures, referrer, postBody) => {
  			e.preventDefault()
			Electron.shell.openExternal(url);
		}
		const handleRedirect = (e, url) => {
			var menu = Electron.Menu.getApplicationMenu();
			menu.items[0].submenu.items[menu.items[0].submenu.items.length - 2].enabled = url.indexOf('settings') === -1;
			Electron.Menu.setApplicationMenu(menu);
			if (url.indexOf('localhost') === -1 || url.indexOf('openbrowser') > -1) {
				e.preventDefault();
				Electron.shell.openExternal(url);
			}
		};

		const view = new Electron.BrowserView()
		w.addBrowserView(view)
		view.setBounds({ x: 420, y: 210, width: 450, height: 450 })
		view.webContents.loadURL(`file://${__dirname}/assets/about.html`);
		view.webContents.on('dom-ready', () => {
			view.webContents.insertCSS('#about-logo{ animation: rotation 4s infinite linear;} @keyframes rotation { from { transform: rotate(0deg); } to { transform: rotate(359deg); } }');
			console.log('CSS was inserted to view');
		});

		w.webContents.on('will-navigate', handleRedirect);
		w.webContents.on('new-window', handleRedirectNewWindow);
		// if loading url failed then try to reload with defined count
		w.webContents.on('did-fail-load', (event, errorCode, errorDescription, validatedURL) => {
			setTimeout(() => {
				reloadCount--;
				if ( reloadCount > 0 )
					return w.reload();
				
				const options = {
				    type: 'error',
				    buttons: [], // Empty Array shows just OK button which has index 0
				    defaultId: 2,
				    title: 'Error',
				    message: 'Loading Graphileon App',
				    detail: 'Error: ' + errorDescription
				};

			  	Electron.dialog.showMessageBoxSync(w, options); // returns clicked button index
				Electron.app.quit(); // after clicked button exit from app
			}, 1000);
		});
		
		w.webContents.on('did-finish-load', () => {
			w.show();
		});

		w.webContents.on('dom-ready', () => {
			w.webContents.insertCSS('a.openBrowser.hide{display: inline-block !important;}');
			console.log('CSS was inserted');
		});

		// `win.loadFile` will escape `#` to `%23`, So use `win.loadURL`
		w.loadURL(`http://localhost:${config.getUserConfig().port}`);
		//remove the loading screen when graphileon title setted
		w.on('page-title-updated', (event, title) => w.removeBrowserView(view) );
		this._windows.set(id, w);

		return w;
	}

	/**
     * Show the about application window.
     */
	createAboutWindow() {
		if (this._aboutDialog) {
			return;
		}
		const top = Electron.BrowserWindow.getFocusedWindow();
		const w = new Electron.BrowserWindow({
			width: 450,
			height: 450,
			resizable: false,
			alwaysOnTop: true,
			modal: false,
			parent: top,
			show: false,
			thickFrame: true,
			maximizable: false,
			minimizable: false,
			title: config.APP_DESC,
			webPreferences: {
				nodeIntegration: true
			}
		});

		w.setMenu(null);

		w.on('closed', () => {
			if (DEBUG) {
				console.log('The about application window was closed.');
			}

			this._aboutDialog = null;
		});

		w.loadFile('assets/about.html');
		this._aboutDialog = w;

		var handleRedirect = (e, url) => {
			if (url.indexOf('localhost') === -1) {
				e.preventDefault();
				Electron.shell.openExternal(url);
			}
		};

		w.webContents.on('will-navigate', handleRedirect);
		w.webContents.on('new-window', handleRedirect);

		w.once('ready-to-show', () => {
		  w.show();
		});
	}
	/**
     * remove the user token for next login
     */
	removeToken() {
		const w = Electron.BrowserWindow.getFocusedWindow();

		if (w) {
			if (DEBUG) {
				console.log('Remove Token');
			}

			var utcDate = new Date(Date.UTC(0, 0, 0, 0, 0, 0));
			let cookie = { url: config.getUserConfig().url, name: 'token', expired: utcDate.toUTCString() };
			w.webContents.session.cookies.set(cookie, err => err ? console.log(err) : '');
		}
	}


	showAlert(message, options) {
		const w = Electron.BrowserWindow.getFocusedWindow();
		if (w) {
			if (DEBUG) {
				console.log('Show Alert');
			}

    		w.webContents.executeJavaScript(`graphileonClient.viewmanager.alert(\`${message}\`, ${JSON.stringify(options)}).then().catch(e => {})`);
    	}
	}
}

module.exports = WindowManager;